﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using System.IO;
using C1.DataEngine;
using C1.PivotEngine;
using C1.AdoNet.Salesforce;
using System.Data.SqlClient;
using System.ComponentModel;

namespace FlexPivotEngine
{
    public class DataEngineService
    {
        #region properties

        private Workspace _workSpace;
        public Workspace Workspace => _workSpace;
        private dynamic invoice;
        private readonly string QueryName = "AllQuery";
        public C1PivotEngine engine;


        #endregion

        #region ctor

        public DataEngineService()
        {
            Init();
        }

        #endregion

        #region Single Column
        public IBindingList SalesByCountry()
        {
            var fp = this.CreateFlexPivotEngine(this.QueryName, new string[] { "" }, new string[] { "Country__c" }, new string[] { "Sales" });
            return fp.PivotDefaultView;
        }        

        public IBindingList ProductQuantitySold()
        {
            var fp = this.CreateFlexPivotEngine(this.QueryName, new string[] { "" }, new string[] { "ProductName__c" }, new string[] { "Quantity" });
            return fp.PivotDefaultView;
        }

        #endregion

        #region Multi Column
        public IBindingList ProductSalesByCountry()
        {
            var fp = this.CreateFlexPivotEngine(this.QueryName, new string[] { "Country__c" }, new string[] { "ProductName__c" }, new string[] { "Sales" });
            return fp.PivotDefaultView;
        }

        public IBindingList QuantitySoldByPerson()
        {
            var fp = this.CreateFlexPivotEngine(this.QueryName, new string[] { "Country__c" }, new string[] { "ProductName__c", "Salesperson__c" }, new string[] { "QuantityTotal" });
            return fp.PivotDefaultView;
        }

        #endregion

        #region Range Groups

        public IBindingList SalesGroupByYear()
        {
            var fp = this.CreateFlexPivotEngine(this.QueryName, new string[] { }, new string[] { "OrderDate__c" }, new string[] { "Sales" });
            var fld = fp.RowFields[0];
            fld.Range.RangeType = RangeType.Format;
            fld.Format = "yyyy";
            fld.Caption = "Year";
            return fp.PivotDefaultView;
        }

        public IBindingList ProductsSoldInYearQuarterly()
        {
            var fp = this.CreateFlexPivotEngine(this.QueryName, new string[] { }, new string[] { "OrderDate__c", "OrderDate__c" }, new string[] { "QuantityTotal" });
            var year = fp.RowFields[0];
            year.Range.RangeType = RangeType.Format;
            year.Format = "yyyy";
            year.Caption = "Year";

            var month = fp.RowFields[1];
            month.Range.RangeType = RangeType.FiscalQuarter;
            
            return fp.PivotDefaultView;
        }

        #endregion

        #region Filtering

        public IBindingList FilterESalesByCountry()
        {
            var fp = this.CreateFlexPivotEngine(this.QueryName, new string[] { }, new string[] { "Country__c", "ProductName__c" }, new string[] { "Sales" });
            var fld = fp.RowFields[0].Filter;
            fld.Condition1.Operator = ConditionOperator.Equals;
            fld.Condition1.Parameter = "Brazil";
            return fp.PivotDefaultView;
        }

        public IBindingList FilterNESalesByCountry()
        {
            var fp = this.CreateFlexPivotEngine(this.QueryName, new string[] { }, new string[] { "Country__c", "ProductName__c" }, new string[] { "Sales" });
            var fld = fp.RowFields[0].Filter;
            fld.Condition1.Operator = ConditionOperator.NotEquals;
            fld.Condition1.Parameter = "Brazil";
            return fp.PivotDefaultView;
        }

        public IBindingList FilteredSalesByCountry()
        {
            var fp = this.CreateFlexPivotEngine(this.QueryName, new string[] { }, new string[] { "Country__c" }, new string[] { "Sales" });
            var fld = fp.ValueFields[0].Filter;
            fld.Condition1.Operator = ConditionOperator.GreaterThanOrEqualTo;
            fld.Condition1.Parameter = 100;
            fld.Condition2.Operator = ConditionOperator.LessThanOrEqualTo;
            fld.Condition2.Parameter = 200;
            fld.AndConditions = true;
            return fp.PivotDefaultView;
        }

        #endregion

        #region Helper methods
        private void Init()
        {
            this._workSpace = new Workspace();
            this._workSpace.Init("workspace");

            C1SalesforceConnection conn = new C1SalesforceConnection(GetConnectionString());
            conn.Open();

            var command = new C1SalesforceCommand(conn, "Select * from Invoices__c");

            var connector = new DbConnector(this._workSpace, conn, command);
            connector.GetData("Invoices__c");

            this.invoice = this._workSpace.table("Invoices__c");

            var queryName = this.QueryName;
            var settings = new
            {
                invoice.Country__c,
                invoice.ProductName__c,
                invoice.Salesperson__c,
                invoice.OrderDate__c,
                Sales = Op.Sum(invoice.ExtendedPrice__c),
                Quantity = Op.Count(invoice.Quantity__c),
                QuantityTotal = Op.Sum(invoice.Quantity__c)
            };

            this.ExecuteQuery(queryName, settings);
            this.engine = new C1PivotEngine();

            this.engine.Workspace = this._workSpace;
            this.engine.ConnectDataEngine(queryName);            
        }

        private C1PivotEngine CreateFlexPivotEngine(string queryName, string[] colFields, string[] rowFields, string[] valFields)
        {
            var fp = this.engine;           
            fp.BeginUpdate();
            this.engine.ColumnFields.Clear();
            this.engine.RowFields.Clear();
            this.engine.ValueFields.Clear();
            fp.ColumnFields.Add(colFields);
            fp.RowFields.Add(rowFields);
            fp.ValueFields.Add(valFields);
            fp.EndUpdate();
            return fp;
        }

        private void ExecuteQuery(string queryName, object settings)
        {
            dynamic query = GetQuery(queryName, settings);
            query.Query.Execute();
        }        

        private dynamic GetQuery(string queryName, object settings)
        {
            return this._workSpace.query(queryName, settings);
        }

        private dynamic GetQuery(object settings)
        {
            return this._workSpace.query(settings);
        }

        private string GetConnectionString()
        {
            const string GCSalesforceServerConnectionString = @"Username=*****;Password=*****;Security Token=*****;
                                                            OAuth Client Id=*****; 
                                                            OAuth Client Secret=*****; 
                                                            OAuth Token Endpoint=https://ap16.salesforce.com/services/oauth2/token; 
                                                            Url=https://ap16.salesforce.com/services/data/v42.0";
            return GCSalesforceServerConnectionString;
        }
        #endregion
    }

}
